@@ -25,21 +25,23 @@ from castellan.common.objects import managed_object
 class OpaqueData(managed_object.ManagedObject):
     """This class represents opaque data."""
 
-    def __init__(self, data, name=None, created=None):
+    def __init__(self, data, name=None, created=None, id=None):
         """Create a new OpaqueData object.
 
         Expected type for data is a bytestring.
         """
         self._data = data
-        super(OpaqueData, self).__init__(name=name, created=created)
+        super().__init__(name=name, created=created, id=id)
+
+    @classmethod
+    def managed_type(cls):
+        return "opaque"
 
     @property
     def format(self):
-        """This method returns 'Opaque'."""
         return "Opaque"
 
     def get_encoded(self):
-        """Returns the data in its original format."""
         return self._data
 
     def __eq__(self, other):
