@@ -25,21 +25,23 @@ from castellan.common.objects import certificate
 class X509(certificate.Certificate):
     """This class represents X.509 certificates."""
 
-    def __init__(self, data, name=None, created=None):
+    def __init__(self, data, name=None, created=None, id=None):
         """Create a new X509 object.
 
         The data should be in a bytestring.
         """
         self._data = data
-        super(X509, self).__init__(name=name, created=created)
+        super().__init__(name=name, created=created, id=id)
+
+    @classmethod
+    def managed_type(cls):
+        return "certificate"
 
     @property
     def format(self):
-        """This method returns 'X.509'."""
         return "X.509"
 
     def get_encoded(self):
-        """Returns the data in its encoded format."""
         return self._data
 
     def __eq__(self, other):
