@@ -18,12 +18,14 @@ Key manager implementation for Barbican
 """
 import calendar
 import time
+import urllib
 
 from cryptography.hazmat import backends
 from cryptography.hazmat.primitives import serialization
 from cryptography import x509 as cryptography_x509
-from keystoneclient.auth import identity
-from keystoneclient import session
+from keystoneauth1 import identity
+from keystoneauth1 import loading
+from keystoneauth1 import session
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
@@ -31,28 +33,25 @@ from oslo_utils import excutils
 from castellan.common import exception
 from castellan.common.objects import key as key_base_class
 from castellan.common.objects import opaque_data as op_data
-from castellan.common.objects import passphrase
-from castellan.common.objects import private_key as pri_key
-from castellan.common.objects import public_key as pub_key
-from castellan.common.objects import symmetric_key as sym_key
-from castellan.common.objects import x_509
+from castellan.i18n import _
 from castellan.key_manager import key_manager
-from castellan.openstack.common import _i18n as u
 
-from barbicanclient import client as barbican_client
+
+from barbicanclient import client as barbican_client_import
 from barbicanclient import exceptions as barbican_exceptions
 from oslo_utils import timeutils
-from six.moves import urllib
 
 
-barbican_opts = [
+_barbican_opts = [
     cfg.StrOpt('barbican_endpoint',
                help='Use this endpoint to connect to Barbican, for example: '
                     '"http://localhost:9311/"'),
     cfg.StrOpt('barbican_api_version',
                help='Version of the Barbican API, for example: "v1"'),
     cfg.StrOpt('auth_endpoint',
-               default='http://localhost:5000/v3',
+               default='http://localhost/identity/v3',
+               deprecated_name='auth_url',
+               deprecated_group='key_manager',
                help='Use this endpoint to connect to Keystone'),
     cfg.IntOpt('retry_delay',
                default=1,
@@ -62,9 +61,26 @@ barbican_opts = [
                default=60,
                help='Number of times to retry poll for key creation '
                     'completion'),
+    cfg.BoolOpt('verify_ssl',
+                default=True,
+                help='Specifies if insecure TLS (https) requests. If False, '
+                     'the server\'s certificate will not be validated, if '
+                     'True, we can set the verify_ssl_path config meanwhile.'),
+    cfg.StrOpt('verify_ssl_path',
+               default=None,
+               help='A path to a bundle or CA certs to check against, or '
+                    'None for requests to attempt to locate and use '
+                    'certificates which verify_ssh is True. If verify_ssl '
+                    'is False, this is ignored.'),
+    cfg.StrOpt('barbican_endpoint_type',
+               default='public',
+               choices=['public', 'internal', 'admin'],
+               help='Specifies the type of endpoint.  Allowed values are: '
+                    'public, private, and admin'),
+
 ]
 
-BARBICAN_OPT_GROUP = 'barbican'
+_BARBICAN_OPT_GROUP = 'barbican'
 
 LOG = logging.getLogger(__name__)
 
@@ -72,20 +88,12 @@ LOG = logging.getLogger(__name__)
 class BarbicanKeyManager(key_manager.KeyManager):
     """Key Manager Interface that wraps the Barbican client API."""
 
-    _secret_type_dict = {
-        op_data.OpaqueData: 'opaque',
-        passphrase.Passphrase: 'passphrase',
-        pri_key.PrivateKey: 'private',
-        pub_key.PublicKey: 'public',
-        sym_key.SymmetricKey: 'symmetric',
-        x_509.X509: 'certificate'}
-
     def __init__(self, configuration):
         self._barbican_client = None
         self._base_url = None
         self.conf = configuration
-        self.conf.register_opts(barbican_opts, group=BARBICAN_OPT_GROUP)
-        session.Session.register_conf_options(self.conf, BARBICAN_OPT_GROUP)
+        self.conf.register_opts(_barbican_opts, group=_BARBICAN_OPT_GROUP)
+        loading.register_session_conf_options(self.conf, _BARBICAN_OPT_GROUP)
 
     def _get_barbican_client(self, context):
         """Creates a client to connect to the Barbican service.
@@ -99,7 +107,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
 
         # Confirm context is provided, if not raise forbidden
         if not context:
-            msg = u._("User is not authorized to use key manager.")
+            msg = _("User is not authorized to use key manager.")
             LOG.error(msg)
             raise exception.Forbidden(msg)
 
@@ -108,16 +116,21 @@ class BarbicanKeyManager(key_manager.KeyManager):
 
         try:
             auth = self._get_keystone_auth(context)
-            sess = session.Session(auth=auth)
+            verify_ssl = self.conf.barbican.verify_ssl
+            verify_ssl_path = self.conf.barbican.verify_ssl_path
+            verify = verify_ssl and verify_ssl_path or verify_ssl
+            sess = session.Session(auth=auth, verify=verify)
 
             self._barbican_endpoint = self._get_barbican_endpoint(auth, sess)
-            self._barbican_client = barbican_client.Client(
+            self._barbican_client = barbican_client_import.Client(
                 session=sess,
                 endpoint=self._barbican_endpoint)
             self._current_context = context
 
+        # TODO(pbourke): more fine grained exception handling - we are eating
+        # tracebacks here
         except Exception as e:
-            LOG.error(u._LE("Error creating Barbican client: %s"), e)
+            LOG.error("Error creating Barbican client: %s", e)
             raise exception.KeyManagerError(reason=e)
 
         self._base_url = self._create_base_url(auth,
@@ -127,11 +140,9 @@ class BarbicanKeyManager(key_manager.KeyManager):
         return self._barbican_client
 
     def _get_keystone_auth(self, context):
-        auth_url = self.conf.barbican.auth_endpoint
-
-        if context.__class__.__name__ is 'KeystonePassword':
-            return identity.v3.Password(
-                auth_url=auth_url,
+        if context.__class__.__name__ == 'KeystonePassword':
+            return identity.Password(
+                auth_url=context.auth_url,
                 username=context.username,
                 password=context.password,
                 user_id=context.user_id,
@@ -145,9 +156,9 @@ class BarbicanKeyManager(key_manager.KeyManager):
                 project_domain_id=context.project_domain_id,
                 project_domain_name=context.project_domain_name,
                 reauthenticate=context.reauthenticate)
-        elif context.__class__.__name__ is 'KeystoneToken':
-            return identity.v3.Token(
-                auth_url=auth_url,
+        elif context.__class__.__name__ == 'KeystoneToken':
+            return identity.Token(
+                auth_url=context.auth_url,
                 token=context.token,
                 trust_id=context.trust_id,
                 domain_id=context.domain_id,
@@ -159,42 +170,63 @@ class BarbicanKeyManager(key_manager.KeyManager):
                 reauthenticate=context.reauthenticate)
         # this will be kept for oslo.context compatibility until
         # projects begin to use utils.credential_factory
-        elif context.__class__.__name__ is 'RequestContext':
-            return identity.v3.Token(
-                auth_url=auth_url,
-                token=context.auth_token,
-                project_id=context.tenant)
+        elif context.__class__.__name__ == 'RequestContext':
+            if getattr(context, 'get_auth_plugin', None):
+                return context.get_auth_plugin()
+            else:
+                return identity.Token(
+                    auth_url=self.conf.barbican.auth_endpoint,
+                    token=context.auth_token,
+                    project_id=context.project_id,
+                    project_name=context.project_name,
+                    project_domain_id=context.project_domain_id,
+                    project_domain_name=context.project_domain_name)
         else:
-            msg = "context must be of type KeystonePassword, KeystoneToken, "
-            "or RequestContext."
+            msg = _("context must be of type KeystonePassword, "
+                    "KeystoneToken, or RequestContext.")
             LOG.error(msg)
             raise exception.Forbidden(reason=msg)
 
     def _get_barbican_endpoint(self, auth, sess):
-        if self.conf.barbican.barbican_endpoint:
-            return self.conf.barbican.barbican_endpoint
+        barbican = self.conf.barbican
+        if barbican.barbican_endpoint:
+            return barbican.barbican_endpoint
+        elif getattr(auth, 'service_catalog', None):
+            endpoint_data = auth.service_catalog.endpoint_data_for(
+                service_type='key-manager',
+                interface=barbican.barbican_endpoint_type)
+            return endpoint_data.url
         else:
             service_parameters = {'service_type': 'key-manager',
                                   'service_name': 'barbican',
-                                  'interface': 'public'}
+                                  'interface': barbican.barbican_endpoint_type}
             return auth.get_endpoint(sess, **service_parameters)
 
     def _create_base_url(self, auth, sess, endpoint):
+        api_version = None
         if self.conf.barbican.barbican_api_version:
             api_version = self.conf.barbican.barbican_api_version
-        else:
+        elif getattr(auth, 'service_catalog', None):
+            endpoint_data = auth.service_catalog.endpoint_data_for(
+                service_type='key-manager')
+            api_version = endpoint_data.api_version
+        elif getattr(auth, 'get_discovery', None):
             discovery = auth.get_discovery(sess, url=endpoint)
             raw_data = discovery.raw_version_data()
             if len(raw_data) == 0:
-                msg = u._LE(
+                msg = _(
                     "Could not find discovery information for %s") % endpoint
                 LOG.error(msg)
                 raise exception.KeyManagerError(reason=msg)
             latest_version = raw_data[-1]
             api_version = latest_version.get('id')
 
+        if endpoint[-1] != '/':
+            endpoint += '/'
+
         base_url = urllib.parse.urljoin(
             endpoint, api_version)
+
         return base_url
 
     def create_key(self, context, algorithm, length,
@@ -224,7 +256,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         except (barbican_exceptions.HTTPAuthError,
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
-            LOG.error(u._LE("Error creating key: %s"), e)
+            LOG.error("Error creating key: %s", e)
             raise exception.KeyManagerError(reason=e)
 
     def create_key_pair(self, context, algorithm, length,
@@ -262,7 +294,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         except (barbican_exceptions.HTTPAuthError,
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
-            LOG.error(u._LE("Error creating key pair: %s"), e)
+            LOG.error("Error creating key pair: %s", e)
             raise exception.KeyManagerError(reason=e)
 
     def _get_barbican_object(self, barbican_client, managed_object):
@@ -341,7 +373,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         except (barbican_exceptions.HTTPAuthError,
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
-            LOG.error(u._LE("Error storing object: %s"), e)
+            LOG.error("Error storing object: %s", e)
             raise exception.KeyManagerError(reason=e)
 
     def _create_secret_ref(self, object_id):
@@ -351,7 +383,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         :return: the URL of the requested secret
         """
         if not object_id:
-            msg = "Key ID is None"
+            msg = _("Key ID is None")
             raise exception.KeyManagerError(reason=msg)
         base_url = self._base_url
         if base_url[-1] != '/':
@@ -375,8 +407,8 @@ class BarbicanKeyManager(key_manager.KeyManager):
                 kwargs = {"status": error_status,
                           "code": order.error_status_code,
                           "reason": order.error_reason}
-                msg = u._LE("Order is in %(status)s status - status code: "
-                            "%(code)s, status reason: %(reason)s") % kwargs
+                msg = _("Order is in %(status)s status - status code: "
+                        "%(code)s, status reason: %(reason)s") % kwargs
                 LOG.error(msg)
                 raise exception.KeyManagerError(reason=msg)
             if order.status != active_status:
@@ -385,17 +417,17 @@ class BarbicanKeyManager(key_manager.KeyManager):
                           'status': order.status,
                           'active': active_status,
                           'delay': retry_delay}
-                msg = u._LI("Retry attempt #%(attempt)i out of %(total)i: "
-                            "Order status is '%(status)s'. Waiting for "
-                            "'%(active)s', will retry in %(delay)s "
-                            "seconds")
+                msg = _("Retry attempt #%(attempt)i out of %(total)i: "
+                        "Order status is '%(status)s'. Waiting for "
+                        "'%(active)s', will retry in %(delay)s "
+                        "seconds")
                 LOG.info(msg, kwargs)
                 time.sleep(retry_delay)
                 order = barbican_client.orders.get(order_ref)
             else:
                 return order
-        msg = u._LE("Exceeded retries: Failed to find '%(active)s' status "
-                    "within %(num_retries)i retries") % {
+        msg = _("Exceeded retries: Failed to find '%(active)s' status "
+                "within %(num_retries)i retries") % {
             'active': active_status,
             'num_retries': number_of_retries}
         LOG.error(msg)
@@ -448,10 +480,17 @@ class BarbicanKeyManager(key_manager.KeyManager):
         else:
             return secret.payload
 
-    def _get_castellan_object(self, secret):
+    def _get_castellan_object(self, secret, metadata_only=False):
         """Creates a Castellan managed object given the Barbican secret.
 
-        :param secret: the secret from barbican with the payload of data
+        The python barbicanclient lazy-loads the secret data, i.e. the secret
+        data is not requested until secret.payload is called. If the user
+        specifies metadata_only=True, secret.payload is never called,
+        preventing unnecessary loading of secret data.
+
+        :param secret: the barbican secret object
+        :metadata_only: boolean indicating if the secret bytes should be
+                        included in the managed object
         :returns: the castellan object
         """
         secret_type = op_data.OpaqueData
@@ -459,7 +498,15 @@ class BarbicanKeyManager(key_manager.KeyManager):
             if barbican_type == secret.secret_type:
                 secret_type = castellan_type
 
-        secret_data = self._get_secret_data(secret)
+        if metadata_only:
+            secret_data = None
+        else:
+            secret_data = self._get_secret_data(secret)
+
+        if secret.secret_ref:
+            object_id = self._retrieve_secret_uuid(secret.secret_ref)
+        else:
+            object_id = None
 
         # convert created ISO8601 in Barbican to POSIX
         if secret.created:
@@ -472,11 +519,13 @@ class BarbicanKeyManager(key_manager.KeyManager):
                                secret.bit_length,
                                secret_data,
                                secret.name,
-                               created)
+                               created,
+                               object_id)
         else:
             return secret_type(secret_data,
                                secret.name,
-                               created)
+                               created,
+                               object_id)
 
     def _get_secret(self, context, object_id):
         """Returns the metadata of the secret.
@@ -499,7 +548,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
             with excutils.save_and_reraise_exception():
-                LOG.error(u._LE("Error getting secret metadata: %s"), e)
+                LOG.error("Error getting secret metadata: %s", e)
 
     def _is_secret_not_found_error(self, error):
         if (isinstance(error, barbican_exceptions.HTTPClientError) and
@@ -508,25 +557,24 @@ class BarbicanKeyManager(key_manager.KeyManager):
         else:
             return False
 
-    def get(self, context, managed_object_id):
+    def get(self, context, managed_object_id, metadata_only=False):
         """Retrieves the specified managed object.
 
-        Currently only supports retrieving symmetric keys.
-
         :param context: contains information of the user and the environment
                         for the request (castellan/context.py)
         :param managed_object_id: the UUID of the object to retrieve
-        :return: SymmetricKey representation of the key
+        :param metadata_only: whether secret data should be included
+        :return: ManagedObject representation of the managed object
         :raises KeyManagerError: if object retrieval fails
         :raises ManagedObjectNotFoundError: if object not found
         """
         try:
             secret = self._get_secret(context, managed_object_id)
-            return self._get_castellan_object(secret)
+            return self._get_castellan_object(secret, metadata_only)
         except (barbican_exceptions.HTTPAuthError,
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
-            LOG.error(u._LE("Error retrieving object: %s"), e)
+            LOG.error("Error retrieving object: %s", e)
             if self._is_secret_not_found_error(e):
                 raise exception.ManagedObjectNotFoundError(
                     uuid=managed_object_id)
@@ -539,7 +587,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         :param context: contains information of the user and the environment
                      for the request (castellan/context.py)
         :param managed_object_id: the UUID of the object to delete
-        :raises KeyManagerError: if key deletion fails
+        :raises KeyManagerError: if object deletion fails
         :raises ManagedObjectNotFoundError: if the object could not be found
         """
         barbican_client = self._get_barbican_client(context)
@@ -550,9 +598,53 @@ class BarbicanKeyManager(key_manager.KeyManager):
         except (barbican_exceptions.HTTPAuthError,
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
-            LOG.error(u._LE("Error deleting object: %s"), e)
+            LOG.error("Error deleting object: %s", e)
             if self._is_secret_not_found_error(e):
                 raise exception.ManagedObjectNotFoundError(
                     uuid=managed_object_id)
             else:
                 raise exception.KeyManagerError(reason=e)
+
+    def list(self, context, object_type=None, metadata_only=False):
+        """Retrieves a list of managed objects that match the criteria.
+
+        If no search criteria is given, all objects are returned.
+
+        :param context: contains information of the user and the environment
+                     for the request (castellan/context.py)
+        :param object_type: the type of object to retrieve
+        :param metadata_only: whether secret data should be included
+        :raises KeyManagerError: if listing secrets fails
+        """
+        objects = []
+        barbican_client = self._get_barbican_client(context)
+
+        if object_type and object_type not in self._secret_type_dict:
+            msg = _("Invalid secret type: %s") % object_type
+            LOG.error(msg)
+            raise exception.KeyManagerError(reason=msg)
+
+        secret_type = self._secret_type_dict.get(object_type)
+
+        try:
+            secrets = barbican_client.secrets.list(secret_type=secret_type)
+        except (barbican_exceptions.HTTPAuthError,
+                barbican_exceptions.HTTPClientError,
+                barbican_exceptions.HTTPServerError) as e:
+            LOG.error("Error listing objects: %s", e)
+            raise exception.KeyManagerError(reason=e)
+
+        for secret in secrets:
+            try:
+                obj = self._get_castellan_object(secret, metadata_only)
+                objects.append(obj)
+            except (barbican_exceptions.HTTPAuthError,
+                    barbican_exceptions.HTTPClientError,
+                    barbican_exceptions.HTTPServerError) as e:
+                LOG.warning("Error occurred while retrieving object "
+                            "metadata, not adding it to the list: %s", e)
+
+        return objects
+
+    def list_options_for_discovery(self):
+        return [(_BARBICAN_OPT_GROUP, _barbican_opts)]
