@@ -29,28 +29,39 @@ import uuid
 from keystoneauth1 import loading
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import timeutils
 import requests
-import six
 
 from castellan.common import exception
-from castellan.common.objects import opaque_data as op_data
-from castellan.common.objects import passphrase
 from castellan.common.objects import private_key as pri_key
 from castellan.common.objects import public_key as pub_key
 from castellan.common.objects import symmetric_key as sym_key
-from castellan.common.objects import x_509
 from castellan.i18n import _
 from castellan.key_manager import key_manager
 
-DEFAULT_VAULT_URL = "http://127.0.0.1:8200"
+_DEFAULT_VAULT_URL = "http://127.0.0.1:8200"
+_DEFAULT_MOUNTPOINT = "secret"
+_DEFAULT_VERSION = 2
 
-vault_opts = [
+_vault_opts = [
     cfg.StrOpt('root_token_id',
                help='root token for vault'),
+    cfg.StrOpt('approle_role_id',
+               help='AppRole role_id for authentication with vault'),
+    cfg.StrOpt('approle_secret_id',
+               help='AppRole secret_id for authentication with vault'),
+    cfg.StrOpt('kv_mountpoint',
+               default=_DEFAULT_MOUNTPOINT,
+               help='Mountpoint of KV store in Vault to use, for example: '
+                    '{}'.format(_DEFAULT_MOUNTPOINT)),
+    cfg.IntOpt('kv_version',
+               default=_DEFAULT_VERSION,
+               help='Version of KV store in Vault to use, for example: '
+                    '{}'.format(_DEFAULT_VERSION)),
     cfg.StrOpt('vault_url',
-               default=DEFAULT_VAULT_URL,
+               default=_DEFAULT_VAULT_URL,
                help='Use this endpoint to connect to Vault, for example: '
-                    '"%s"' % DEFAULT_VAULT_URL),
+                    '"%s"' % _DEFAULT_VAULT_URL),
     cfg.StrOpt('ssl_ca_crt_file',
                help='Absolute path to ca cert file'),
     cfg.BoolOpt('use_ssl',
@@ -58,7 +69,7 @@ vault_opts = [
                 help=_('SSL Enabled/Disabled')),
 ]
 
-VAULT_OPT_GROUP = 'vault'
+_VAULT_OPT_GROUP = 'vault'
 
 _EXCEPTIONS_BY_CODE = [
     requests.codes['internal_server_error'],
@@ -74,20 +85,19 @@ LOG = logging.getLogger(__name__)
 class VaultKeyManager(key_manager.KeyManager):
     """Key Manager Interface that wraps the Vault REST API."""
 
-    _secret_type_dict = {
-        op_data.OpaqueData: 'opaque',
-        passphrase.Passphrase: 'passphrase',
-        pri_key.PrivateKey: 'private',
-        pub_key.PublicKey: 'public',
-        sym_key.SymmetricKey: 'symmetric',
-        x_509.X509: 'certificate'}
-
     def __init__(self, configuration):
         self._conf = configuration
-        self._conf.register_opts(vault_opts, group=VAULT_OPT_GROUP)
-        loading.register_session_conf_options(self._conf, VAULT_OPT_GROUP)
+        self._conf.register_opts(_vault_opts, group=_VAULT_OPT_GROUP)
+        loading.register_session_conf_options(self._conf, _VAULT_OPT_GROUP)
         self._session = requests.Session()
         self._root_token_id = self._conf.vault.root_token_id
+        self._approle_role_id = self._conf.vault.approle_role_id
+        self._approle_secret_id = self._conf.vault.approle_secret_id
+        self._cached_approle_token_id = None
+        self._approle_token_ttl = None
+        self._approle_token_issue = None
+        self._kv_mountpoint = self._conf.vault.kv_mountpoint
+        self._kv_version = self._conf.vault.kv_version
         self._vault_url = self._conf.vault.vault_url
         if self._vault_url.startswith("https://"):
             self._verify_server = self._conf.vault.ssl_ca_crt_file or True
@@ -99,15 +109,94 @@ class VaultKeyManager(key_manager.KeyManager):
             self._vault_url += '/'
         return self._vault_url
 
+    def _get_resource_url(self, key_id=None):
+        return '{}v1/{}/{}{}'.format(
+            self._get_url(),
+            self._kv_mountpoint,
+
+            '' if self._kv_version == 1 else
+            'data/' if key_id else
+            'metadata/',  # no key_id is for listing and 'data/' doesn't works
+
+            key_id if key_id else '?list=true')
+
+    @property
+    def _approle_token_id(self):
+        if (all((self._approle_token_issue, self._approle_token_ttl)) and
+                timeutils.is_older_than(self._approle_token_issue,
+                                        self._approle_token_ttl)):
+            self._cached_approle_token_id = None
+        return self._cached_approle_token_id
+
+    def _build_auth_headers(self):
+        if self._root_token_id:
+            return {'X-Vault-Token': self._root_token_id}
+
+        if self._approle_token_id:
+            return {'X-Vault-Token': self._approle_token_id}
+
+        if self._approle_role_id:
+            params = {
+                'role_id': self._approle_role_id
+            }
+            if self._approle_secret_id:
+                params['secret_id'] = self._approle_secret_id
+            approle_login_url = '{}v1/auth/approle/login'.format(
+                self._get_url()
+            )
+            token_issue_utc = timeutils.utcnow()
+            try:
+                resp = self._session.post(url=approle_login_url,
+                                          json=params,
+                                          verify=self._verify_server)
+            except requests.exceptions.Timeout as ex:
+                raise exception.KeyManagerError(str(ex))
+            except requests.exceptions.ConnectionError as ex:
+                raise exception.KeyManagerError(str(ex))
+            except Exception as ex:
+                raise exception.KeyManagerError(str(ex))
+
+            if resp.status_code in _EXCEPTIONS_BY_CODE:
+                raise exception.KeyManagerError(resp.reason)
+            if resp.status_code == requests.codes['forbidden']:
+                raise exception.Forbidden()
+
+            resp_data = resp.json()
+
+            if resp.status_code == requests.codes['bad_request']:
+                raise exception.KeyManagerError(', '.join(resp_data['errors']))
+
+            self._cached_approle_token_id = resp_data['auth']['client_token']
+            self._approle_token_issue = token_issue_utc
+            self._approle_token_ttl = resp_data['auth']['lease_duration']
+            return {'X-Vault-Token': self._approle_token_id}
+
+        return {}
+
+    def _do_http_request(self, method, resource, json=None):
+        verify = self._verify_server
+        headers = self._build_auth_headers()
+
+        try:
+            resp = method(resource, headers=headers, json=json, verify=verify)
+        except requests.exceptions.Timeout as ex:
+            raise exception.KeyManagerError(str(ex))
+        except requests.exceptions.ConnectionError as ex:
+            raise exception.KeyManagerError(str(ex))
+        except Exception as ex:
+            raise exception.KeyManagerError(str(ex))
+
+        if resp.status_code in _EXCEPTIONS_BY_CODE:
+            raise exception.KeyManagerError(resp.reason)
+        if resp.status_code == requests.codes['forbidden']:
+            raise exception.Forbidden()
+
+        return resp
+
     def create_key_pair(self, context, algorithm, length,
                         expiration=None, name=None):
         """Creates an asymmetric key pair."""
 
-        # Confirm context is provided, if not raise forbidden
-        if not context:
-            msg = _("User is not authorized to use key manager.")
-            raise exception.Forbidden(msg)
-
         if algorithm.lower() != 'rsa':
             raise NotImplementedError(
                 "VaultKeyManager only implements rsa keys"
@@ -157,97 +246,64 @@ class VaultKeyManager(key_manager.KeyManager):
             raise exception.KeyManagerError(
                 "Unknown type for value : %r" % value)
 
-        headers = {'X-Vault-Token': self._root_token_id}
-        try:
-            resource_url = self._get_url() + 'v1/secret/' + key_id
-            record = {
-                'type': type_value,
-                'value': binascii.hexlify(value.get_encoded()).decode('utf-8'),
-                'algorithm': (value.algorithm if hasattr(value, 'algorithm')
-                              else None),
-                'bit_length': (value.bit_length if hasattr(value, 'bit_length')
-                               else None),
-                'name': value.name,
-                'created': value.created
-            }
-            resp = self._session.post(resource_url,
-                                      verify=self._verify_server,
-                                      json=record,
-                                      headers=headers)
-        except requests.exceptions.Timeout as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
-        except requests.exceptions.ConnectionError as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
-        except Exception as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
+        record = {
+            'type': type_value,
+            'value': binascii.hexlify(value.get_encoded()).decode('utf-8'),
+            'algorithm': (value.algorithm if hasattr(value, 'algorithm')
+                          else None),
+            'bit_length': (value.bit_length if hasattr(value, 'bit_length')
+                           else None),
+            'name': value.name,
+            'created': value.created
+        }
+        if self._kv_version > 1:
+            record = {'data': record}
 
-        if resp.status_code in _EXCEPTIONS_BY_CODE:
-            raise exception.KeyManagerError(resp.reason)
-        if resp.status_code == requests.codes['forbidden']:
-            raise exception.Forbidden()
+        self._do_http_request(self._session.post,
+                              self._get_resource_url(key_id),
+                              json=record)
 
         return key_id
 
     def create_key(self, context, algorithm, length, name=None, **kwargs):
         """Creates a symmetric key."""
 
-        # Confirm context is provided, if not raise forbidden
-        if not context:
-            msg = _("User is not authorized to use key manager.")
-            raise exception.Forbidden(msg)
+        if length % 8:
+            msg = _("Length must be multiple of 8.")
+            raise ValueError(msg)
 
         key_id = uuid.uuid4().hex
-        key_value = os.urandom(length or 32)
+        key_value = os.urandom((length or 256) // 8)
         key = sym_key.SymmetricKey(algorithm,
-                                   length or 32,
+                                   length or 256,
                                    key_value,
                                    key_id,
                                    name or int(time.time()))
+
         return self._store_key_value(key_id, key)
 
     def store(self, context, key_value, **kwargs):
         """Stores (i.e., registers) a key with the key manager."""
 
-        # Confirm context is provided, if not raise forbidden
-        if not context:
-            msg = _("User is not authorized to use key manager.")
-            raise exception.Forbidden(msg)
-
         key_id = uuid.uuid4().hex
         return self._store_key_value(key_id, key_value)
 
     def get(self, context, key_id, metadata_only=False):
         """Retrieves the key identified by the specified id."""
 
-        # Confirm context is provided, if not raise forbidden
-        if not context:
-            msg = _("User is not authorized to use key manager.")
-            raise exception.Forbidden(msg)
-
         if not key_id:
             raise exception.KeyManagerError('key identifier not provided')
 
-        headers = {'X-Vault-Token': self._root_token_id}
-        try:
-            resource_url = self._get_url() + 'v1/secret/' + key_id
-            resp = self._session.get(resource_url,
-                                     verify=self._verify_server,
-                                     headers=headers)
-        except requests.exceptions.Timeout as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
-        except requests.exceptions.ConnectionError as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
-        except Exception as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
+        resp = self._do_http_request(self._session.get,
+                                     self._get_resource_url(key_id))
 
-        if resp.status_code in _EXCEPTIONS_BY_CODE:
-            raise exception.KeyManagerError(resp.reason)
-        if resp.status_code == requests.codes['forbidden']:
-            raise exception.Forbidden()
         if resp.status_code == requests.codes['not_found']:
             raise exception.ManagedObjectNotFoundError(uuid=key_id)
 
         record = resp.json()['data']
+        if self._kv_version > 1:
+            record = record['data']
+
         key = None if metadata_only else binascii.unhexlify(record['value'])
 
         clazz = None
@@ -275,66 +331,29 @@ class VaultKeyManager(key_manager.KeyManager):
     def delete(self, context, key_id):
         """Represents deleting the key."""
 
-        # Confirm context is provided, if not raise forbidden
-        if not context:
-            msg = _("User is not authorized to use key manager.")
-            raise exception.Forbidden(msg)
-
         if not key_id:
             raise exception.KeyManagerError('key identifier not provided')
 
-        headers = {'X-Vault-Token': self._root_token_id}
-        try:
-            resource_url = self._get_url() + 'v1/secret/' + key_id
-            resp = self._session.delete(resource_url,
-                                        verify=self._verify_server,
-                                        headers=headers)
-        except requests.exceptions.Timeout as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
-        except requests.exceptions.ConnectionError as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
-        except Exception as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
+        resp = self._do_http_request(self._session.delete,
+                                     self._get_resource_url(key_id))
 
-        if resp.status_code in _EXCEPTIONS_BY_CODE:
-            raise exception.KeyManagerError(resp.reason)
-        if resp.status_code == requests.codes['forbidden']:
-            raise exception.Forbidden()
         if resp.status_code == requests.codes['not_found']:
             raise exception.ManagedObjectNotFoundError(uuid=key_id)
 
     def list(self, context, object_type=None, metadata_only=False):
         """Lists the managed objects given the criteria."""
 
-        # Confirm context is provided, if not raise forbidden
-        if not context:
-            msg = _("User is not authorized to use key manager.")
-            raise exception.Forbidden(msg)
-
         if object_type and object_type not in self._secret_type_dict:
             msg = _("Invalid secret type: %s") % object_type
             raise exception.KeyManagerError(reason=msg)
 
-        headers = {'X-Vault-Token': self._root_token_id}
-        try:
-            resource_url = self._get_url() + 'v1/secret/?list=true'
-            resp = self._session.get(resource_url,
-                                     verify=self._verify_server,
-                                     headers=headers)
-            keys = resp.json()['data']['keys']
-        except requests.exceptions.Timeout as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
-        except requests.exceptions.ConnectionError as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
-        except Exception as ex:
-            raise exception.KeyManagerError(six.text_type(ex))
+        resp = self._do_http_request(self._session.get,
+                                     self._get_resource_url())
 
-        if resp.status_code in _EXCEPTIONS_BY_CODE:
-            raise exception.KeyManagerError(resp.reason)
-        if resp.status_code == requests.codes['forbidden']:
-            raise exception.Forbidden()
         if resp.status_code == requests.codes['not_found']:
             keys = []
+        else:
+            keys = resp.json()['data']['keys']
 
         objects = []
         for obj_id in keys:
@@ -343,7 +362,10 @@ class VaultKeyManager(key_manager.KeyManager):
                 if object_type is None or isinstance(obj, object_type):
                     objects.append(obj)
             except exception.ManagedObjectNotFoundError as e:
-                LOG.warning(_("Error occurred while retrieving object "
-                              "metadata, not adding it to the list: %s"), e)
+                LOG.warning("Error occurred while retrieving object "
+                            "metadata, not adding it to the list: %s", e)
                 pass
         return objects
+
+    def list_options_for_discovery(self):
+        return [(_VAULT_OPT_GROUP, _vault_opts)]
