@@ -12,10 +12,12 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+from stevedore import ExtensionManager
+
 from oslo_config import cfg
 from oslo_log import log
 
-from castellan import key_manager as km
+from castellan import key_manager
 try:
     from castellan.key_manager import barbican_key_manager as bkm
 except ImportError:
@@ -39,7 +41,10 @@ _DEFAULT_LOGGING_CONTEXT_FORMAT = ('%(asctime)s.%(msecs)03d %(process)d '
 def set_defaults(conf, backend=None, barbican_endpoint=None,
                  barbican_api_version=None, auth_endpoint=None,
                  retry_delay=None, number_of_retries=None, verify_ssl=None,
-                 api_class=None, vault_root_token_id=None, vault_url=None,
+                 verify_ssl_path=None,
+                 api_class=None, vault_root_token_id=None,
+                 vault_approle_role_id=None, vault_approle_secret_id=None,
+                 vault_kv_mountpoint=None, vault_url=None,
                  vault_ssl_ca_crt_file=None, vault_use_ssl=None,
                  barbican_endpoint_type=None):
     """Set defaults for configuration values.
@@ -53,18 +58,28 @@ def set_defaults(conf, backend=None, barbican_endpoint=None,
     :param retry_delay: Use this attribute to set retry delay.
     :param number_of_retries: Use this attribute to set number of retries.
     :param verify_ssl: Use this to specify if ssl should be verified.
+    :param verify_ssl_path: Use this to specify the CA path.
     :param vault_root_token_id: Use this for the root token id for vault.
+    :param vault_approle_role_id: Use this for the approle role_id for vault.
+    :param vault_approle_secret_id: Use this for the approle secret_id
+                                    for vault.
+    :param vault_kv_mountpoint: Mountpoint of KV store in vault to use.
     :param vault_url: Use this for the url for vault.
     :param vault_use_ssl: Use this to force vault driver to use ssl.
     :param vault_ssl_ca_crt_file: Use this for the CA file for vault.
     :param barbican_endpoint_type: Use this to specify the type of URL.
     :                              Valid values are: public, internal or admin.
     """
-    conf.register_opts(km.key_manager_opts, group='key_manager')
-    if bkm:
-        conf.register_opts(bkm.barbican_opts, group=bkm.BARBICAN_OPT_GROUP)
-    if vkm:
-        conf.register_opts(vkm.vault_opts, group=vkm.VAULT_OPT_GROUP)
+    conf.register_opts(key_manager.key_manager_opts, group='key_manager')
+
+    ext_mgr = ExtensionManager(
+        "castellan.drivers",
+        invoke_on_load=True,
+        invoke_args=[cfg.CONF])
+
+    for km in ext_mgr.names():
+        for group, opts in ext_mgr[km].obj.list_options_for_discovery():
+            conf.register_opts(opts, group=group)
 
     # Use the new backend option if set or fall back to the older api_class
     default_backend = backend or api_class
@@ -74,39 +89,51 @@ def set_defaults(conf, backend=None, barbican_endpoint=None,
     if bkm is not None:
         if barbican_endpoint is not None:
             conf.set_default('barbican_endpoint', barbican_endpoint,
-                             group=bkm.BARBICAN_OPT_GROUP)
+                             group=bkm._BARBICAN_OPT_GROUP)
         if barbican_api_version is not None:
             conf.set_default('barbican_api_version', barbican_api_version,
-                             group=bkm.BARBICAN_OPT_GROUP)
+                             group=bkm._BARBICAN_OPT_GROUP)
         if auth_endpoint is not None:
             conf.set_default('auth_endpoint', auth_endpoint,
-                             group=bkm.BARBICAN_OPT_GROUP)
+                             group=bkm._BARBICAN_OPT_GROUP)
         if retry_delay is not None:
             conf.set_default('retry_delay', retry_delay,
-                             group=bkm.BARBICAN_OPT_GROUP)
+                             group=bkm._BARBICAN_OPT_GROUP)
         if number_of_retries is not None:
             conf.set_default('number_of_retries', number_of_retries,
-                             group=bkm.BARBICAN_OPT_GROUP)
+                             group=bkm._BARBICAN_OPT_GROUP)
         if verify_ssl is not None:
             conf.set_default('verify_ssl', verify_ssl,
-                             group=bkm.BARBICAN_OPT_GROUP)
+                             group=bkm._BARBICAN_OPT_GROUP)
+        if verify_ssl_path is not None:
+            conf.set_default('verify_ssl_path', verify_ssl_path,
+                             group=bkm._BARBICAN_OPT_GROUP)
         if barbican_endpoint_type is not None:
             conf.set_default('barbican_endpoint_type', barbican_endpoint_type,
-                             group=bkm.BARBICAN_OPT_GROUP)
+                             group=bkm._BARBICAN_OPT_GROUP)
 
     if vkm is not None:
         if vault_root_token_id is not None:
             conf.set_default('root_token_id', vault_root_token_id,
-                             group=vkm.VAULT_OPT_GROUP)
+                             group=vkm._VAULT_OPT_GROUP)
+        if vault_approle_role_id is not None:
+            conf.set_default('approle_role_id', vault_approle_role_id,
+                             group=vkm._VAULT_OPT_GROUP)
+        if vault_approle_secret_id is not None:
+            conf.set_default('approle_secret_id', vault_approle_secret_id,
+                             group=vkm._VAULT_OPT_GROUP)
+        if vault_kv_mountpoint is not None:
+            conf.set_default('kv_mountpoint', vault_kv_mountpoint,
+                             group=vkm._VAULT_OPT_GROUP)
         if vault_url is not None:
             conf.set_default('vault_url', vault_url,
-                             group=vkm.VAULT_OPT_GROUP)
+                             group=vkm._VAULT_OPT_GROUP)
         if vault_ssl_ca_crt_file is not None:
             conf.set_default('ssl_ca_crt_file', vault_ssl_ca_crt_file,
-                             group=vkm.VAULT_OPT_GROUP)
+                             group=vkm._VAULT_OPT_GROUP)
         if vault_use_ssl is not None:
             conf.set_default('use_ssl', vault_use_ssl,
-                             group=vkm.VAULT_OPT_GROUP)
+                             group=vkm._VAULT_OPT_GROUP)
 
 
 def enable_logging(conf=None, app_name='castellan'):
@@ -136,12 +163,16 @@ def list_opts():
     :returns: a list of (group_name, opts) tuples
     """
     key_manager_opts = []
-    key_manager_opts.extend(km.key_manager_opts)
+    key_manager_opts.extend(key_manager.key_manager_opts)
     key_manager_opts.extend(utils.credential_opts)
     opts = [('key_manager', key_manager_opts)]
 
-    if bkm is not None:
-        opts.append((bkm.BARBICAN_OPT_GROUP, bkm.barbican_opts))
-    if vkm is not None:
-        opts.append((vkm.VAULT_OPT_GROUP, vkm.vault_opts))
+    ext_mgr = ExtensionManager(
+        "castellan.drivers",
+        invoke_on_load=True,
+        invoke_args=[cfg.CONF])
+
+    for driver in ext_mgr.names():
+        opts.extend(ext_mgr[driver].obj.list_options_for_discovery())
+
     return opts
