@@ -17,8 +17,15 @@
 Test cases for the key manager.
 """
 
+from oslo_config import cfg
+from oslo_config import fixture
+
+from castellan import key_manager
+from castellan.key_manager import barbican_key_manager
 from castellan.tests import base
 
+CONF = cfg.CONF
+
 
 class KeyManagerTestCase(base.TestCase):
 
@@ -28,4 +35,18 @@ class KeyManagerTestCase(base.TestCase):
     def setUp(self):
         super(KeyManagerTestCase, self).setUp()
 
+        self.conf = self.useFixture(fixture.Config()).conf
+
         self.key_mgr = self._create_key_manager()
+
+
+class DefaultKeyManagerImplTestCase(KeyManagerTestCase):
+
+    def _create_key_manager(self):
+        return key_manager.API(self.conf)
+
+    def test_default_key_manager(self):
+        self.assertEqual("barbican", self.conf.key_manager.backend)
+        self.assertIsNotNone(self.key_mgr)
+        self.assertIsInstance(self.key_mgr,
+                              barbican_key_manager.BarbicanKeyManager)
