@@ -16,6 +16,7 @@
 """
 Test cases for the opaque data class.
 """
+from castellan.common import objects
 from castellan.common.objects import opaque_data
 from castellan.tests import base
 
@@ -35,6 +36,13 @@ class OpaqueDataTestCase(base.TestCase):
 
         super(OpaqueDataTestCase, self).setUp()
 
+    def test_is_not_only_metadata(self):
+        self.assertFalse(self.opaque_data.is_metadata_only())
+
+    def test_is_only_metadata(self):
+        d = opaque_data.OpaqueData(None, self.name, self.created)
+        self.assertTrue(d.is_metadata_only())
+
     def test_get_format(self):
         self.assertEqual('Opaque', self.opaque_data.format)
 
@@ -60,7 +68,7 @@ class OpaqueDataTestCase(base.TestCase):
         self.assertTrue(self.opaque_data is self.opaque_data)
 
         self.assertFalse(self.opaque_data is None)
-        self.assertFalse(None == self.opaque_data)
+        self.assertFalse(None == self.opaque_data)  # noqa: E711
 
         other_opaque_data = opaque_data.OpaqueData(self.data)
         self.assertTrue(self.opaque_data == other_opaque_data)
@@ -68,8 +76,12 @@ class OpaqueDataTestCase(base.TestCase):
 
     def test___ne___none(self):
         self.assertTrue(self.opaque_data is not None)
-        self.assertTrue(None != self.opaque_data)
+        self.assertTrue(None != self.opaque_data)  # noqa: E711
 
     def test___ne___data(self):
         other_opaque = opaque_data.OpaqueData(b'other data', self.name)
         self.assertTrue(self.opaque_data != other_opaque)
+
+    def test_to_and_from_dict(self):
+        other = objects.from_dict(self.opaque_data.to_dict())
+        self.assertEqual(self.opaque_data, other)
