@@ -16,6 +16,7 @@
 """
 Test cases for the symmetric key class.
 """
+from castellan.common import objects
 from castellan.common.objects import symmetric_key as sym_key
 from castellan.tests import base
 
@@ -38,6 +39,17 @@ class SymmetricKeyTestCase(base.KeyTestCase):
 
         super(SymmetricKeyTestCase, self).setUp()
 
+    def test_is_not_only_metadata(self):
+        self.assertFalse(self.key.is_metadata_only())
+
+    def test_is_only_metadata(self):
+        k = sym_key.SymmetricKey(self.algorithm,
+                                 self.bit_length,
+                                 None,
+                                 self.name,
+                                 self.created)
+        self.assertTrue(k.is_metadata_only())
+
     def test_get_format(self):
         self.assertEqual('RAW', self.key.format)
 
@@ -71,7 +83,7 @@ class SymmetricKeyTestCase(base.KeyTestCase):
         self.assertTrue(self.key is self.key)
 
         self.assertFalse(self.key is None)
-        self.assertFalse(None == self.key)
+        self.assertFalse(None == self.key)  # noqa: E711
 
         other_key = sym_key.SymmetricKey(self.algorithm,
                                          self.bit_length,
@@ -81,7 +93,7 @@ class SymmetricKeyTestCase(base.KeyTestCase):
 
     def test___ne___none(self):
         self.assertTrue(self.key is not None)
-        self.assertTrue(None != self.key)
+        self.assertTrue(None != self.key)  # noqa: E711
 
     def test___ne___algorithm(self):
         other_key = sym_key.SymmetricKey('DES',
@@ -104,3 +116,7 @@ class SymmetricKeyTestCase(base.KeyTestCase):
                                          different_encoded,
                                          self.name)
         self.assertTrue(self.key != other_key)
+
+    def test_to_and_from_dict(self):
+        other = objects.from_dict(self.key.to_dict())
+        self.assertEqual(self.key, other)
