@@ -16,6 +16,7 @@
 """
 Test cases for the X.509 certificate class.
 """
+from castellan.common import objects
 from castellan.common.objects import x_509
 from castellan.tests import base
 from castellan.tests import utils
@@ -35,6 +36,13 @@ class X509TestCase(base.CertificateTestCase):
 
         super(X509TestCase, self).setUp()
 
+    def test_is_not_only_metadata(self):
+        self.assertFalse(self.cert.is_metadata_only())
+
+    def test_is_only_metadata(self):
+        c = x_509.X509(None, self.name, self.created)
+        self.assertTrue(c.is_metadata_only())
+
     def test_get_format(self):
         self.assertEqual('X.509', self.cert.format)
 
@@ -60,7 +68,7 @@ class X509TestCase(base.CertificateTestCase):
         self.assertTrue(self.cert is self.cert)
 
         self.assertFalse(self.cert is None)
-        self.assertFalse(None == self.cert)
+        self.assertFalse(None == self.cert)  # noqa: E711
 
         other_x_509 = x_509.X509(self.data)
         self.assertTrue(self.cert == other_x_509)
@@ -68,8 +76,12 @@ class X509TestCase(base.CertificateTestCase):
 
     def test___ne___none(self):
         self.assertTrue(self.cert is not None)
-        self.assertTrue(None != self.cert)
+        self.assertTrue(None != self.cert)  # noqa: E711
 
     def test___ne___data(self):
         other_x509 = x_509.X509(b'\x00\x00\x00', self.name)
         self.assertTrue(self.cert != other_x509)
+
+    def test_to_and_from_dict(self):
+        other = objects.from_dict(self.cert.to_dict())
+        self.assertEqual(self.cert, other)
